@@ -1,6 +1,6 @@
 
 /*
- *  restore.c  $Id: restore.c,v 2.6 2003/04/21 03:27:50 bkorb Exp $
+ *  restore.c  $Id: restore.c,v 4.1 2005/01/01 00:12:42 bkorb Exp $
  *
  *  This module's routines will save the current option state to memory
  *  and restore it.  If saved prior to the initial optionProcess call,
@@ -8,7 +8,7 @@
  */
 
 /*
- *  Automated Options copyright 1992-2003 Bruce Korb
+ *  Automated Options copyright 1992-2004 Bruce Korb
  *
  *  Automated Options is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -50,6 +50,9 @@
  * If you do not wish that, delete this exception notice.
  */
 
+/* === STATIC PROCS === */
+/* === END STATIC PROCS === */
+
 /*=export_func optionSaveState
  *
  * what:  saves the option state to memory
@@ -58,15 +61,17 @@
  * doc:   This routine will allocate enough memory to save the current
  *        option processing state.  If this routine has been called before,
  *        that memory will be reused.  You may only save one copy of the
- *        option state.  This routine may be called before optionProcess(3).
+ *        option state.  This routine may be called before optionProcess(3AO).
+ *        If you do call it before the first call to optionProcess, then
+ *        you may also change the contents of argc/argv after you call
+ *        optionRestore(3AO)
  *
  * err:   If it fails to allocate the memory,
  *        it will print a message to stderr and exit.
  *        Otherwise, it will always succeed.
 =*/
 void
-optionSaveState( pOpts )
-    tOptions* pOpts;
+optionSaveState( tOptions* pOpts )
 {
     if (pOpts->pSavedState == NULL) {
         size_t sz = sizeof( *pOpts ) + (pOpts->optCt * sizeof( tOptDesc ));
@@ -76,10 +81,9 @@ optionSaveState( pOpts )
             if (pzName == NULL) {
                 pzName = pOpts->pzPROGNAME;
                 if (pzName == NULL)
-                    pzName = "";
+                    pzName = zNil;
             }
-            fprintf( stderr, "%s(optionSaveState): error: cannot allocate "
-                     "%d bytes\n", pzName, sz );
+            fprintf( stderr, zCantSave, pzName, sz );
             exit( EXIT_FAILURE );
         }
     }
@@ -100,13 +104,15 @@ optionSaveState( pOpts )
  * doc:  Copy back the option state from saved memory.
  *       The allocated memory is left intact, so this routine can be
  *       called repeatedly without having to call optionSaveState again.
+ *       If you are restoring a state that was saved before the first call
+ *       to optionProcess(3AO), then you may change the contents of the
+ *       argc/argv parameters to optionProcess.
  *
  * err:  If you have not called @code{optionSaveState} before, a diagnostic is
  *       printed to @code{stderr} and exit is called.
 =*/
 void
-optionRestore( pOpts )
-    tOptions* pOpts;
+optionRestore( tOptions* pOpts )
 {
     tOptions* p = (tOptions*)pOpts->pSavedState;
 
@@ -115,10 +121,9 @@ optionRestore( pOpts )
         if (pzName == NULL) {
             pzName = pOpts->pzPROGNAME;
             if (pzName == NULL)
-                pzName = "";
+                pzName = zNil;
         }
-        fprintf( stderr, "%s(optionRestore): error: no saved option state\n",
-                 pzName );
+        fprintf( stderr, zNoState, pzName );
         exit( EXIT_FAILURE );
     }
     memcpy( pOpts, p, sizeof( *p ));
@@ -139,8 +144,7 @@ optionRestore( pOpts )
  *        this routine is always successful.      
 =*/
 void
-optionFree( pOpts )
-    tOptions* pOpts;
+optionFree( tOptions* pOpts )
 {
     if (pOpts->pSavedState != NULL) {
         AGFREE( pOpts->pSavedState );
@@ -150,7 +154,7 @@ optionFree( pOpts )
         tOptDesc* p = pOpts->pOptDesc;
         int ct = pOpts->optCt;
         do  {
-            if (p->optCookie != NULL) {
+            if ((p->fOptState & OPTST_STACKED) && (p->optCookie != NULL)) {
                 AGFREE( p->optCookie );
                 p->fOptState &= OPTST_PERSISTENT;
                 if ((p->fOptState & OPTST_INITENABLED) == 0)
